// Copyright 1997-2001 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import <OmniAppKit/OAPreferenceClientRecord.h>

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <OmniBase/OmniBase.h>
#import <OmniFoundation/OmniFoundation.h>

#import <OmniAppKit/OAPreferenceClient.h>
#import <OmniAppKit/OAPreferenceController.h>

RCS_ID("$Header: /Network/Source/CVS/OmniGroup/Frameworks/OmniAppKit/Preferences.subproj/OAPreferenceClientRecord.m,v 1.18 2001/08/01 23:50:49 wiml Exp $")

@implementation OAPreferenceClientRecord

- (id)initWithCategoryName:(NSString *)newName;
{
    if (![super init])
        return nil;
    categoryName = [newName retain];
    return self;
}

- (void)dealloc;
{
    [categoryName release];
    [identifier release];
    [className release];
    [title release];
    [shortTitle release];
    [iconName release];
    [nibName release];
    [helpURL release];
    [defaultsDictionary release];
    [defaultsArray release];
    [iconImage release];
    [clientInstance release];
    
    [super dealloc];
}


//

- (NSImage *)iconImage;
{
    NSBundle *bundle;
    NSString *path;
    
    if (iconImage)
        return iconImage;

    bundle = [OFBundledClass bundleForClassNamed:className];
    path = [bundle pathForImageResource:iconName];
    iconImage = [[NSImage alloc] initWithContentsOfFile:path];
    return iconImage;
}

- (NSString *)categoryName;
{
    return categoryName;
}

- (NSString *)identifier;
{
    return identifier;
}

- (NSString *)className;
{
    return className;
}

- (NSString *)title;
{
    return title;
}

- (NSString *)shortTitle;
{
    return shortTitle ? shortTitle : title;
}

- (NSString *)iconName;
{
    return iconName;
}

- (NSString *)nibName;
{
    return nibName;
}

- (NSString *)helpURL;
{
    return helpURL;
}

- (NSNumber *)ordering;
{
    return ordering;
}

- (NSDictionary *)defaultsDictionary;
{
    return defaultsDictionary;
}

- (NSArray *)defaultsArray;
{
    return defaultsArray;
}

- (void)setIdentifier:(NSString *)newIdentifier;
{
    if (identifier == newIdentifier)
        return;
    [identifier release];
    identifier = [newIdentifier retain];
}

- (void)setClassName:(NSString *)newClassName;
{
    if (className == newClassName)
        return;
    [className release];
    className = [newClassName retain];
}

- (void)setTitle:(NSString *)newTitle;
{
    if (title == newTitle)
        return;
    [title release];
    title = [newTitle retain];
}

- (void)setShortTitle:(NSString *)newShortTitle;
{
    if (shortTitle == newShortTitle)
        return;
    [shortTitle release];
    shortTitle = [newShortTitle retain];
}

- (void)setIconName:(NSString *)newIconName;
{
    if (iconName == newIconName)
        return;
    [iconName release];
    iconName = [newIconName retain];
}

- (void)setNibName:(NSString *)newNibName;
{
    if (nibName == newNibName)
        return;
    [nibName release];
    nibName = [newNibName retain];
}

- (void)setHelpURL:(NSString *)newHelpURL;
{
    if (helpURL == newHelpURL)
        return;
    [helpURL release];
    helpURL = [newHelpURL retain];
}

- (void)setOrdering:(NSNumber *)newOrdering;
{
    if (ordering == newOrdering)
        return;
    [ordering release];
    ordering = [newOrdering retain];
}

- (void)setDefaultsDictionary:(NSDictionary *)newDefaultsDictionary;
{
    if (defaultsDictionary == newDefaultsDictionary)
        return;
    [defaultsDictionary release];
    defaultsDictionary = [newDefaultsDictionary retain];
}

- (void)setDefaultsArray:(NSArray *)newDefaultsArray;
{
    if (defaultsArray == newDefaultsArray)
        return;
    [defaultsArray release];
    defaultsArray = [newDefaultsArray retain];
}

- (NSComparisonResult)compare:(OAPreferenceClientRecord *)other;
{
    if (![other isKindOfClass:[self class]])
	return NSOrderedAscending;

    return [[self shortTitle] compare:[other shortTitle]];
}

- (NSComparisonResult)compareOrdering:(OAPreferenceClientRecord *)other;
{
    if (![other isKindOfClass:[self class]])
	return NSOrderedAscending;

    return [[self ordering] compare:[other ordering]];
}

- (OAPreferenceClient *)clientInstanceInController:(OAPreferenceController *)controller;
{
    Class clientClass;

    if (clientInstance)
        return clientInstance;
    
    [controller setTitle:[NSString stringWithFormat:NSLocalizedStringFromTableInBundle(@"Loading %@...", @"OmniAppKit", [OAPreferenceClientRecord bundle], preference bundle loading message format), title]];

    clientClass = [OFBundledClass classNamed:className];
    if (!defaultsArray)
        defaultsArray = [[defaultsDictionary allKeys] retain];
    else {
        [defaultsArray autorelease];
        defaultsArray = [[defaultsArray arrayByAddingObjectsFromArray:[defaultsDictionary allKeys]] retain];
    }
    
    clientInstance = [[clientClass alloc] initWithTitle:title defaultsArray:defaultsArray];

    if (clientInstance != nil && nibName != nil)
        [NSBundle loadNibNamed:nibName owner:clientInstance];
        
    return clientInstance;
}


//
// Debugging
//

- (NSMutableDictionary *) debugDictionary;
{
    NSMutableDictionary *dict;
    
    dict = [super debugDictionary];
    if (className)
        [dict setObject:className forKey:@"className"];
    if (identifier)
        [dict setObject:identifier forKey:@"identifier"];
    if (title)
        [dict setObject:title forKey:@"title"];
    if (shortTitle)
        [dict setObject:shortTitle forKey:@"shortTitle"];
    if (iconName)
        [dict setObject:iconName forKey:@"iconName"];
    if (nibName)
        [dict setObject:nibName forKey:@"nibName"];
    if (defaultsDictionary)
        [dict setObject:defaultsDictionary forKey:@"defaultsDictionary"];
    if (defaultsArray)
        [dict setObject:defaultsArray forKey:@"defaultsArray"];
    if (iconImage)
        [dict setObject:iconImage forKey:@"iconImage"];
    if (clientInstance)
        [dict setObject:clientInstance forKey:@"clientInstance"];
    return dict;
}

@end
