// Copyright 1997-2001 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import <OmniAppKit/OAScrollView.h>

#import <ApplicationServices/ApplicationServices.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <OmniBase/OmniBase.h>
#import <OmniFoundation/OmniFoundation.h>

#import "NSImage-OAExtensions.h"
#import "NSView-OAExtensions.h"
#import "OAApplication.h"
#import "OAPageSelectableDocumentProtocol.h"
#import "OAZoomableViewProtocol.h"

RCS_ID("$Header: /Network/Source/CVS/OmniGroup/Frameworks/OmniAppKit/Widgets.subproj/OAScrollView.m,v 1.38 2001/08/01 23:50:59 wiml Exp $")

@interface OAScrollView (Private)
- (void)_setupScrollView;
- (void)processKeyDownEvent:(NSEvent *)keyDownEvent;
- (void)pageUp:(id)sender;
- (void)pageDown:(id)sender;
- (void)zoomIn:(id)sender;
- (void)zoomOut:(id)sender;
- (void)addOrRemoveScrollersIfNeeded;
- (void)autoScrollTile;
- (NSSize)contentSizeForHorizontalScroller:(BOOL)hasHorizontalScroller verticalScroller:(BOOL)hasVerticalScroller;
@end

@implementation OAScrollView

#if defined(RHAPSODY) || defined(WIN32)
#define SCROLLERS_ON_RIGHT
#endif

static int startingScales[] = {50, 75, 100, 125, 150, 200, 400, 0};
static NSFont *smallSystemFont;

+ (void)initialize;
{
    OBINITIALIZE;

    smallSystemFont = [NSFont systemFontOfSize:10.0];
}

- initWithFrame:(NSRect)theFrame;
{
    if ([super initWithFrame:theFrame] == nil)
        return nil;
    [self _setupScrollView];
    // Default scroller settings for OAScrollViews
    [self setHasHorizontalScroller:YES];
    [self setHasVerticalScroller:YES];
    return self;
}

- (id)initWithCoder:(NSCoder *)coder;
{
    if ([super initWithCoder:coder] == nil)
        return nil;
    [self _setupScrollView];
    return self;
}

- (void)dealloc;
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    [self removeFromSuperview];
    [horizontalWidgetsBox removeFromSuperview];
    [horizontalWidgetsBox release];
    [pageUpButton removeFromSuperview];
    [pageUpButton release];
    [pageDownButton removeFromSuperview];
    [pageDownButton release];
    [scalePopUpButton removeFromSuperview];
    [scalePopUpButton release];
    [pagePromptTextField removeFromSuperview];
    [pagePromptTextField release];
    [pageNumberTextField removeFromSuperview];
    [pageNumberTextField release];
    [pagesCountTextField removeFromSuperview];
    [pagesCountTextField release];

    [super dealloc];
}

- (void)zoomToScale:(double)newZoomFactor;
{
    if (newZoomFactor == zoomFactor)
        return;

    zoomFactor = newZoomFactor;
    [[self documentView] zoomTo:newZoomFactor];
    [[self documentView] displayIfNeeded];
}

- (void)zoomFromSender:(NSMenuItem *)sender;
{
    double newZoomFactor;

    // This hack is needed under 4.2.  Maybe Rhapsody is better.
    if ([sender isKindOfClass:[NSMatrix class]])
        sender = [(NSMatrix *)sender selectedCell];

    newZoomFactor = [sender tag] / 100.0;
    [self zoomToScale:newZoomFactor];
}

- (float)zoomFactor;
{
    return zoomFactor;
}

- (void)setDelegate:(id)newDelegate;
{
    delegate = newDelegate;

    if ([delegate conformsToProtocol:@protocol(OAPageSelectableDocument)]) {
	NSRect                      textRect;

	textRect = [[self horizontalScroller] frame];

        pagePromptTextField = [[NSTextField alloc] initWithFrame:textRect];
	[pagePromptTextField setFont:smallSystemFont];
	[pagePromptTextField setStringValue:NSLocalizedStringFromTableInBundle(@"Page", @"OmniAppKit", [OAScrollView bundle], page prompt for multipage documents in scrollview)];
	[pagePromptTextField setAlignment:NSRightTextAlignment];
	[pagePromptTextField setBackgroundColor:[NSColor controlColor]];
        [pagePromptTextField setBezeled:NO];
	[pagePromptTextField setEditable:NO];
	[pagePromptTextField setSelectable:NO];
	[horizontalWidgetsBox addSubview:pagePromptTextField];
	
        pageNumberTextField = [[NSTextField alloc] initWithFrame:textRect];
	[pageNumberTextField setFont:smallSystemFont];
	[pageNumberTextField setAlignment:NSCenterTextAlignment];
        [pageNumberTextField setBezeled:NO];
        [pageNumberTextField setBordered:YES];
	[pageNumberTextField setTarget:self];
	[pageNumberTextField setAction:@selector(gotoPage:)];
	[pageNumberTextField setNextResponder:delegate];
        [pageNumberTextField setRefusesFirstResponder:YES];
	[horizontalWidgetsBox addSubview:pageNumberTextField];

        pagesCountTextField = [[NSTextField alloc] initWithFrame:textRect];
	[pagesCountTextField setFont:smallSystemFont];
	[pagesCountTextField setAlignment:NSLeftTextAlignment];
        [pagesCountTextField setBackgroundColor:[NSColor controlColor]];
	[pagesCountTextField setBezeled:NO];
	[pagesCountTextField setEditable:NO];
	[pagesCountTextField setSelectable:NO];
	[horizontalWidgetsBox addSubview:pagesCountTextField];
    } else {
	[pagePromptTextField removeFromSuperview];
	[pagePromptTextField release];
	pagePromptTextField = nil;
	[pageNumberTextField removeFromSuperview];
	[pageNumberTextField release];
	pageNumberTextField = nil;
	[pagesCountTextField removeFromSuperview];
	[pagesCountTextField release];
	pagesCountTextField = nil;
    }

    [self tile];
}

- (void)setScrollBehavior:(ScrollingBehavior)behavior;
{
    scrollBehavior = behavior;
    switch (scrollBehavior) {
        case YES_SCROLL:
            [self setHasHorizontalScroller:YES];
            [self setHasVerticalScroller:YES];
            break;
        case NO_SCROLL:
            [self setHasHorizontalScroller:NO];
            [self setHasVerticalScroller:NO];
            break;
        case AUTO_SCROLL:
            break;
    }
}

- (void)showingPageNumber:(int)pageNumber of:(unsigned int)pagesCount;
{
    if (pageNumber < 0)
        [pageNumberTextField setStringValue:@""];
    else
        [pageNumberTextField setIntValue:pageNumber + 1];
    [pagesCountTextField setStringValue:[NSString stringWithFormat:NSLocalizedStringFromTableInBundle(@"of %d", @"OmniAppKit", [OAScrollView bundle], how many pages in document format for scrollview), pagesCount]];
    [pageNumberTextField setNextResponder:[self documentView]];
}

- (void)gotoPage:(id)sender;
{
    [delegate displayPageNumber:[sender intValue] - 1];
    [pageNumberTextField setNextResponder:[self documentView]];
}

- (BOOL)processKeyDownCharacter:(unichar)character modifierFlags:(unsigned int)modifierFlags;
{
    enum {
        UnicharDeleteKey  = 0x007F,
        UnicharNonBreakingSpaceKey  = 0x00A0
    };

    [NSCursor setHiddenUntilMouseMoves:YES];
    switch (character) {
        case NSUpArrowFunctionKey:
            if (modifierFlags & NSAlternateKeyMask)
                [self scrollDownByPages:-1.0];
            else
                [self scrollDownByLines:-1.0];
            return YES;
        case NSDownArrowFunctionKey:
            if (modifierFlags & NSAlternateKeyMask)
                [self scrollDownByPages:1.0];
            else
                [self scrollDownByLines:1.0];
            return YES;
        case NSLeftArrowFunctionKey:
            if (modifierFlags & NSAlternateKeyMask)
                [self scrollRightByPages:-1.0];
            else
                [self scrollRightByLines:-1.0];
            return YES;
        case NSRightArrowFunctionKey:
            if (modifierFlags & NSAlternateKeyMask)
                [self scrollRightByPages:1.0];
            else
                [self scrollRightByLines:1.0];
            return YES;
        case NSPageUpFunctionKey:
            [self scrollDownByPages:-1.0];
            return YES;
        case NSPageDownFunctionKey:
            [self scrollDownByPages:1.0];
            return YES;
        case NSHomeFunctionKey:
            if (modifierFlags & NSShiftKeyMask)
                [self scrollToEnd];
            else
                [self scrollToTop];
            return YES;
        case NSEndFunctionKey:
            [self scrollToEnd];
            return YES;
        case UnicharDeleteKey:
        case UnicharNonBreakingSpaceKey: // Alt-Space
            [self scrollDownByPages:-1.0];
            return YES;
        case ' ':
            if (modifierFlags & NSShiftKeyMask)
                [self scrollDownByPages:-1.0];
            else
                [self scrollDownByPages:1.0];
            return YES;
        case 'u':
            [self scrollDownByPages:-0.5];
            return YES;
        case 'd':
            [self scrollDownByPages:0.5];
            return YES;
        case 'f':
            [self pageDown:nil];
            return YES;
        case 'b':
            [self pageUp:nil];
            return YES;
        case '[':
            [self zoomIn:nil];
            return YES;
        case ']':
            [self zoomOut:nil];
            return YES;
        default:
            return NO;
    }
}

//

- (void)setSmoothScrollEnabled:(BOOL)smoothScrollEnabled;
{
    flags.smoothScrollDisabled = !smoothScrollEnabled;
}

- (BOOL)smoothScrollEnabled;
{
    return !flags.smoothScrollDisabled;
}

- (NSSize)idealSizeForAvailableSize:(NSSize)availableSize;
{
    NSClipView *clipView;
    NSView *docView;
    NSSize docViewSize;
    NSSize scrollViewSize;
    BOOL hasHorizontalScroller, hasVerticalScroller;

    clipView = [self contentView];
    docView = [clipView documentView];
    if (docView == nil)
        docViewSize = NSZeroSize;
    else if ([docView respondsToSelector:@selector(idealSizeForAvailableSize:)])
        docViewSize = [(id)docView idealSizeForAvailableSize:[isa contentSizeForFrameSize:availableSize hasHorizontalScroller:scrollBehavior == YES_SCROLL hasVerticalScroller:scrollBehavior == YES_SCROLL borderType:[self borderType]]];
    else
        docViewSize = [docView frame].size;

    switch (scrollBehavior) {
        case YES_SCROLL:
            hasHorizontalScroller = YES;
            hasVerticalScroller = YES;
            break;
        case NO_SCROLL:
            hasHorizontalScroller = NO;
            hasVerticalScroller = NO;
            break;
        default:
        case AUTO_SCROLL:
            scrollViewSize = [isa frameSizeForContentSize:docViewSize hasHorizontalScroller:NO hasVerticalScroller:NO borderType:[self borderType]];
            hasVerticalScroller = scrollViewSize.height > availableSize.height;
            scrollViewSize = [isa frameSizeForContentSize:docViewSize hasHorizontalScroller:NO hasVerticalScroller:hasVerticalScroller borderType:[self borderType]];
            hasHorizontalScroller = scrollViewSize.width > availableSize.width;
            break;
    }
    scrollViewSize = [isa frameSizeForContentSize:docViewSize hasHorizontalScroller:hasHorizontalScroller hasVerticalScroller:hasVerticalScroller borderType:[self borderType]];
    return scrollViewSize;
}

// NSScrollView subclass

- (void)setDocumentView:(NSView *)aView
{
    if ([aView conformsToProtocol:@protocol(OAZoomableView)]) {
	unsigned int scaleIndex;

	/* create scale scalePopUpButton */
	scalePopUpButton = [[NSPopUpButton alloc] init];
        [scalePopUpButton setBordered:NO];
        [scalePopUpButton setFont:[NSFont systemFontOfSize:10]];
	for (scaleIndex = 0; startingScales[scaleIndex] != 0; scaleIndex++) {
	    NSString *title;
	    NSMenuItem *scaleCell;

	    title = [NSString stringWithFormat:@"%d%%", startingScales[scaleIndex]];
	    [scalePopUpButton addItemWithTitle:title];
            scaleCell = [scalePopUpButton itemWithTitle:title];
	    [scaleCell setTag:startingScales[scaleIndex]];
	    [scaleCell setTarget:self];
	    [scaleCell setAction:@selector(zoomFromSender:)];
	}

	zoomFactor = 1.0;
	[scalePopUpButton selectItemWithTitle:@"100%"];
        [scalePopUpButton setRefusesFirstResponder:YES];
	[horizontalWidgetsBox addSubview:scalePopUpButton];
    } else {
	[scalePopUpButton removeFromSuperview];
	[scalePopUpButton release];
	scalePopUpButton = nil;
    }

    [super setDocumentView:aView];
    [self addOrRemoveScrollersIfNeeded];
}

- (void)tile;
{
    NSRect scrollerRect, widgetRect;
    NSRect widgetsAreaRect, widgetsBoxRect;
    BOOL hasMultiplePages;
    NSClipView *clipView;
    NSView *docView;

    if (flags.tiling)
        return;
    flags.tiling = YES;

    clipView = [self contentView];
    docView = [clipView documentView];

    if (scrollBehavior == AUTO_SCROLL)
        [self autoScrollTile];
    else
        [super tile];

    // Set up widgets in horizontal scroller
    hasMultiplePages = NSHeight([docView frame]) > [self contentSize].height || [delegate conformsToProtocol:@protocol(OAPageSelectableDocument)];
    if (![self hasHorizontalScroller]) {
        [horizontalWidgetsBox removeFromSuperview];
    } else {
        if (![horizontalWidgetsBox superview])
            [self addSubview:horizontalWidgetsBox];
        scrollerRect = [[self horizontalScroller] frame];
        widgetsBoxRect = scrollerRect;
        widgetsAreaRect = NSMakeRect(0, 0, scrollerRect.size.width, scrollerRect.size.height);

        if (scalePopUpButton) {
            NSDivideRect(widgetsAreaRect, &widgetRect, &widgetsAreaRect, 80.0, NSMinXEdge);
#ifndef RHAPSODY
            widgetRect = NSInsetRect(widgetRect, 1.0, 1.0);
#else
            widgetRect = NSInsetRect(widgetRect, 1.0, -1.0);
#endif
            [scalePopUpButton setFrame:widgetRect];
        }

        if (pagePromptTextField && hasMultiplePages) {
            NSDivideRect(widgetsAreaRect, &widgetRect, &widgetsAreaRect, 39, NSMinXEdge);
            widgetRect = NSInsetRect(widgetRect, 1.0, 0.0);
#ifndef RHAPSODY
            widgetRect.origin.y -= 3.0;
#else
            widgetRect.origin.y -= 1.0;
#endif
            [pagePromptTextField setFrame:widgetRect];

            NSDivideRect(widgetsAreaRect, &widgetRect, &widgetsAreaRect, 37, NSMinXEdge);
            widgetRect = NSInsetRect(widgetRect, 1.0, 0.0);
#ifndef RHAPSODY
            widgetRect.origin.y += 1.0;
            widgetRect.size.height -= 1.0;
#else
            widgetRect.origin.y -= 1.0;
            widgetRect.size.height += 2.0;
#endif
            [pageNumberTextField setFrame:widgetRect];

            NSDivideRect(widgetsAreaRect, &widgetRect, &widgetsAreaRect, 40, NSMinXEdge);
            widgetRect = NSInsetRect(widgetRect, 1.0, 0.0);
#ifndef RHAPSODY
            widgetRect.origin.y -= 3.0;
#else
            widgetRect.origin.y -= 1.0;
#endif
            [pagesCountTextField setFrame:widgetRect];
        }

        scrollerRect.size.width -= NSMinX(widgetsAreaRect);
        [[self horizontalScroller] setFrame:scrollerRect];

        widgetsBoxRect.size.width = NSMinX(widgetsAreaRect);
        widgetsBoxRect.origin.x = NSMaxX(scrollerRect);
        [horizontalWidgetsBox setFrame:widgetsBoxRect];
    }

    // Set up widgets in vertical scroller

    if (![self hasVerticalScroller] || !pageUpButton || !hasMultiplePages) {
        [pageDownButton removeFromSuperview];
        [pageUpButton removeFromSuperview];
    } else {
        NSSize widgetSize;

        widgetSize = NSMakeSize(16.0, 16.0);

        if (![pageUpButton superview])
            [self addSubview:pageUpButton];
        if (![pageDownButton superview])
            [self addSubview:pageDownButton];

        scrollerRect = [[self verticalScroller] frame];

        // lop off the size we want, plus a pixel for spacing below
        NSDivideRect(scrollerRect, &widgetRect, &scrollerRect, widgetSize.height + 1.0, NSMaxYEdge);
        widgetRect.size = widgetSize;
        widgetRect = NSOffsetRect(widgetRect, 1.0, 0.0);
        [pageDownButton setFrame:widgetRect];

        NSDivideRect(scrollerRect, &widgetRect, &scrollerRect, widgetSize.height + 1.0, NSMaxYEdge);
        widgetRect.size = widgetSize;
        widgetRect = NSOffsetRect(widgetRect, 1.0, 0.0);
        [pageUpButton setFrame:widgetRect];

        [[self verticalScroller] setFrame:scrollerRect];
    }

    [self setNeedsDisplay:YES];
    flags.tiling = NO;
}

- (void)reflectScrolledClipView:(NSClipView *)aClipView;
{
    [super reflectScrolledClipView:aClipView];
    [self addOrRemoveScrollersIfNeeded];
}

#if 0
#warning We MUST look at the difference in visible rects here and if small enough just call super.  This is because this method gets called for when we are using fine controls like the scroller knob or buttons.
#warning Alternatively, if the difference in rects is on the order of the scroll quanta, we could use a much lower time interval, or just scroll one pixel at a time.

- (void)scrollClipView:(NSClipView *)aClipView toPoint:(NSPoint)aPoint;
{
    NSRect oldVisibleRect, newVisibleRect, overlapRect, newImageRect;
    NSBitmapImageRep *oldImageRep, *newImageRep;
    NSView *documentView;
    OAImageSlideDirection direction;
    
    if (flags.smoothScrollDisabled) {
        [super scrollClipView:aClipView toPoint:aPoint];
        return;
    }
    
    oldVisibleRect = [self documentVisibleRect];

    // Figure out what direction we are moving.  If we are moving less
    // than 10% of the length of the visible region along that axis,
    // we won't bother doing smooth scrolling.
    
    if (oldVisibleRect.origin.x == aPoint.x) {
        if (fabs(oldVisibleRect.origin.y - aPoint.y / oldVisibleRect.size.height) < 0.10) {
            [super scrollClipView: aClipView toPoint: aPoint];
            return;
        }
        
        if (oldVisibleRect.origin.y < aPoint.y) {
            direction = OAUpSlideDirection;
        } else
            direction = OADownSlideDirection;
    } else if (oldVisibleRect.origin.y == aPoint.y) {
        if (fabs(oldVisibleRect.origin.x - aPoint.x / oldVisibleRect.size.width) < 0.10) {
            [super scrollClipView: aClipView toPoint: aPoint];
            return;
        }

        if (oldVisibleRect.origin.x < aPoint.x) {
            direction = OALeftSlideDirection;
        } else
            direction = OARightSlideDirection;
    } else {
        // We're going diagonal or something ... just pop for now.
        [super scrollClipView: aClipView toPoint: aPoint];
        return;
    }
    
    documentView = [self documentView];
    //NSLog(@"oldVisibleRect = %@", NSStringFromRect(oldVisibleRect));
    
    // Cache the old image
    oldImageRep = [documentView bitmapForRect: oldVisibleRect];
    //[[oldImageRep TIFFRepresentation] writeToFile: @"/tmp/old.tiff" atomically: YES];
    
    // Do the scroll
    [super scrollClipView: aClipView toPoint: aPoint];
    
    // Cache the new image.  If the two rects overlap, we only want to cache
    // the non-overlapping part of the new image.
    newVisibleRect = [self documentVisibleRect];
    //NSLog(@"newVisibleRect = %@", NSStringFromRect(newVisibleRect));

    overlapRect = NSIntersectionRect(oldVisibleRect, newVisibleRect);
    //NSLog(@"overlapRect = %@", NSStringFromRect(overlapRect));

    switch (direction) {
        case OALeftSlideDirection:
            newImageRect = newVisibleRect;
            newImageRect.origin.x += NSWidth(overlapRect);
            newImageRect.size.width -= NSWidth(overlapRect);
            break;
        case OARightSlideDirection:
            newImageRect = newVisibleRect;
            newImageRect.size.width -= NSWidth(overlapRect);
            break;
        case OAUpSlideDirection:
            newImageRect = newVisibleRect;
            newImageRect.origin.y += NSHeight(overlapRect);
            newImageRect.size.height -= NSHeight(overlapRect);
            break;
        case OADownSlideDirection:
            newImageRect = newVisibleRect;
            newImageRect.size.height -= NSHeight(overlapRect);
            break;
    }
    //NSLog(@"newImageRect = %@", NSStringFromRect(newImageRect));
    
    newImageRep = [documentView bitmapForRect: newImageRect];
    //[[newImage TIFFRepresentation] writeToFile: @"/tmp/new.tiff" atomically: YES];
    //NSLog(@"newImage size = %@", NSStringFromSize([newImage size]));
    
    [documentView slideOutOldBitmapImageRep: oldImageRep newBitmapImageRep: newImageRep slideDirection: direction];
}
#endif

// NSResponder subclass

- (BOOL)acceptsFirstResponder;
{
    return YES;
}

- (void)keyDown:(NSEvent *)theEvent;
{
    if (pageNumberTextField) {
        NSString *characters;

        characters = [theEvent characters];
        if ([characters length] > 0) {
            unichar keyDownCharacter;

            keyDownCharacter = [[theEvent characters] characterAtIndex:0];
            if (keyDownCharacter >= '0' && keyDownCharacter <= '9') {
                [pageNumberTextField selectText:nil];
                [[[self window] firstResponder] keyDown:theEvent];
                return;
            }
        }
    }

    [self processKeyDownEvent:theEvent];

    while (YES) {
        // Peek at the next event
        theEvent = [NSApp nextEventMatchingMask:NSAnyEventMask untilDate:[NSDate distantPast] inMode:NSEventTrackingRunLoopMode dequeue:NO];
        // Break the loop if there is no next event
        if (!theEvent)
            break;
        // Skip over key-up events
        else if ([theEvent type] == NSKeyUp) {
            [super keyUp:[NSApp nextEventMatchingMask:NSAnyEventMask untilDate:[NSDate distantPast] inMode:NSEventTrackingRunLoopMode dequeue:YES]];
            continue;
        }
        // Respond only to key-down events
        else if ([theEvent type] == NSKeyDown) {
            [self processKeyDownEvent:[NSApp nextEventMatchingMask:NSAnyEventMask untilDate:[NSDate distantPast] inMode:NSEventTrackingRunLoopMode dequeue:YES]];
        }
        // Break the loop on all other event types
        else
            break;
    }

    [self displayIfNeeded];
#warning Need to collapse keyboard input events here
    // Used to call:
    // PSWait();
}

// NSView subclass

- (BOOL)isOpaque;
{
    return YES;
}

- (void)drawRect:(NSRect)rect;
{
    CGContextRef context;
    NSRect aRect;
    BOOL somethingToDraw = NO;
    float border;

    context = [[NSGraphicsContext currentContext] graphicsPort];

    switch ([self borderType]) {
    default:
    case NSNoBorder:
	border = 0.0;
        [[NSColor controlColor] set];
	NSRectFill(rect);
	break;
    case NSBezelBorder:
	border = 2.0;
	NSDrawDarkBezel([self bounds], rect);
	break;
    case NSLineBorder:
	border = 1.0;
        [[NSColor controlColor] set];
	NSRectFill(rect);
        [[NSColor controlDarkShadowColor] set];
	NSFrameRect([self bounds]);
	break;
    case NSGrooveBorder:
	border = 2.0;
	NSDrawGroove([self bounds], rect);
	break;
    }

    [super drawRect:rect];

    if ([self hasHorizontalScroller]) {
	aRect = [[self horizontalScroller] frame];
	if (!NSIsEmptyRect(NSIntersectionRect(aRect, rect))) {
	    somethingToDraw = YES;
	    CGContextMoveToPoint(context, NSMinX(aRect) + border, NSMinY(aRect) - 1.0);
	    CGContextAddLineToPoint(context, NSMaxX(aRect) - 2.0 * border, NSMinY(aRect) - 1.0);
	}
    }
    if ([self hasVerticalScroller]) {
	aRect = [[self verticalScroller] frame];
	if (!NSIsEmptyRect(NSIntersectionRect(aRect, rect))) {
	    somethingToDraw = YES;
#ifndef SCROLLERS_ON_RIGHT
            CGContextMoveToPoint(context, NSMaxX(aRect) + 1.0, NSMinY(aRect) + border);
            CGContextAddLineToPoint(context, NSMaxX(aRect) + 1.0, NSMaxY(aRect) - 2.0 * border);
#else
            CGContextMoveToPoint(context, NSMinX(aRect) - 1.0, NSMinY(aRect) + border);
            CGContextAddLineToPoint(context, NSMinX(aRect) - 1.0, NSMaxY(aRect) - 2.0 * border);
#endif
        }
    }
    if (somethingToDraw) {
        [[NSColor controlDarkShadowColor] set];
        CGContextStrokePath(context);
    }
}

@end

@implementation OAScrollView (Private)

- (void)_setupScrollView;
{
    // Scroll more quickly
    [self setLineScroll:36.0];

    // Preserve some context when scrolling
    [self setVerticalPageScroll:20.0];
    [self setHorizontalPageScroll:20.0];

    scrollBehavior = AUTO_SCROLL;
    horizontalWidgetsBox = [[NSView alloc] initWithFrame:NSZeroRect];
    [[self contentView] setAutoresizesSubviews:YES];
    [self addSubview:horizontalWidgetsBox];
}

- (void)processKeyDownEvent:(NSEvent *)keyDownEvent;
{
    NSString *characters;
    unsigned int modifierFlags;
    unsigned int characterIndex, characterCount;
    BOOL processedAtLeastOneCharacter = NO;

    [NSCursor setHiddenUntilMouseMoves:YES];
    characters = [keyDownEvent characters];
    modifierFlags = [keyDownEvent modifierFlags];
    characterCount = [characters length];
    for (characterIndex = 0; characterIndex < characterCount; characterIndex++) {
        if ([self processKeyDownCharacter:[characters characterAtIndex:characterIndex] modifierFlags:modifierFlags])
            processedAtLeastOneCharacter = YES;
    }
    if (!processedAtLeastOneCharacter)
        [super keyDown:keyDownEvent];
}

- (void)pageUp:(id)sender;
{
    if ([delegate conformsToProtocol:@protocol(OAPageSelectableDocument)])
        [delegate pageUp];
    else
        [self scrollDownByPages:-1.0];
}

- (void)pageDown:(id)sender;
{
    if ([delegate conformsToProtocol:@protocol(OAPageSelectableDocument)])
        [delegate pageDown];
    else
        [self scrollDownByPages:1.0];
}

- (void)zoomIn:(id)sender;
{
    unsigned int zoomIndex;

    if (!scalePopUpButton)
        return;

    for (zoomIndex = 0; startingScales[zoomIndex] > 0; zoomIndex++) {
        if (zoomFactor * 100.0 < startingScales[zoomIndex]) {
            [scalePopUpButton selectItemWithTitle:[NSString stringWithFormat:@"%d%%", startingScales[zoomIndex]]];
            [self zoomToScale:startingScales[zoomIndex] / 100.0];
            break;
        }
    }
}

- (void)zoomOut:(id)sender;
{
    unsigned int zoomIndex;

    if (!scalePopUpButton)
        return;

    for (zoomIndex = 1; startingScales[zoomIndex] > 0; zoomIndex++) {
        if (zoomFactor * 100.0 <= startingScales[zoomIndex]) {
            [scalePopUpButton selectItemWithTitle:[NSString stringWithFormat:@"%d%%", startingScales[zoomIndex - 1]]];
            [self zoomToScale:startingScales[zoomIndex - 1] / 100.0];
            break;
        }
    }
}

- (void)addOrRemoveScrollersIfNeeded;
{
    if (scrollBehavior == AUTO_SCROLL) {
        NSRect docViewFrame;
        NSSize potentialContentSize;
        BOOL needsVerticalScroller, needsHorizontalScroller;

        docViewFrame = [[self documentView] frame];
        potentialContentSize = [self contentSizeForHorizontalScroller:NO verticalScroller:YES];
        needsVerticalScroller = NSHeight(docViewFrame) > potentialContentSize.height;
        if (!needsVerticalScroller) {
            potentialContentSize = [self contentSizeForHorizontalScroller:NO verticalScroller:NO];
        }
        needsHorizontalScroller = NSWidth(docViewFrame) > potentialContentSize.width;
        if ([self hasVerticalScroller] != needsVerticalScroller ||
            [self hasHorizontalScroller] != needsHorizontalScroller) {
            [self tile];
            [self setNeedsDisplayInRect:[self bounds]];
        }
    }
}

- (void)autoScrollTile;
{
    BOOL needsVerticalScroller, needsHorizontalScroller;
    NSClipView *clipView;
    NSView *docView;

    clipView = [self contentView];
    docView = [clipView documentView];

    if (!docView) {
        if ([self hasVerticalScroller])
            [self setHasVerticalScroller:NO];
        if ([self hasHorizontalScroller])
            [self setHasHorizontalScroller:NO];
        [super tile];
        return;
    }
    
    [super tile];

    needsVerticalScroller = NSHeight([docView frame]) > [self contentSizeForHorizontalScroller:NO verticalScroller:YES].height;
    if (needsVerticalScroller != [self hasVerticalScroller]) {
#if 0
        NSLog(@"%@ needsVerticalScroller? %.1f > %.1f = %d", OBShortObjectDescription(self), NSHeight([docView frame]), [self contentSizeForHorizontalScroller:NO verticalScroller:YES].height, needsVerticalScroller);
#endif
        [self setHasVerticalScroller:needsVerticalScroller];
        [super tile];
    }

    needsHorizontalScroller = NSWidth([docView frame]) > [self contentSize].width;
    if (needsHorizontalScroller != [self hasHorizontalScroller]) {
#if 0
        NSLog(@"%@ needsHorizontalScroller? %.1f > %.1f = %d", OBShortObjectDescription(self), NSWidth([docView frame]), [self contentSize].width, needsHorizontalScroller);
#endif
        [self setHasHorizontalScroller:needsHorizontalScroller];
        [super tile];
    }
}

- (NSSize)contentSizeForHorizontalScroller:(BOOL)hasHorizontalScroller verticalScroller:(BOOL)hasVerticalScroller;
{
    return [isa contentSizeForFrameSize:[self frame].size hasHorizontalScroller:hasHorizontalScroller hasVerticalScroller:hasVerticalScroller borderType:[self borderType]];
}

@end
